﻿//===================================================================
//  pas1000s.cs
//
//  Copyright (C) P&A Technologies Inc.
//===================================================================
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace Sample
{
    public partial class pas1000s : Form
    {
        #region "定数"
        // 分解能
        private const double DATA_RESOLUTION = 65536;
        // USB転送モード
        private readonly string[] STR_USB_MODE = { "FULL SPEED", "HI SPEED" };
        // 曜日
        private readonly string[] STR_WEEK = { "日", "月", "火", "水", "木", "金", "土" };
        // 動作モード
        private readonly string[] STR_ENABLE = { "無効", "有効" };
        private readonly string[] STR_ANALOG_TYPE = { "電圧", "", "", "", "", "", "", "", "", "測温抵抗体" };
        private readonly string[] STR_ANALOG_RANGE = { "±100mV", "±200mV", "±500mV", "±1V", "±2V", "±5V",
                                                         "±10V", "±20V", "±50V", "±100V" };
        private readonly string[] STR_ANALOG_RANGE_UNIT = { "mV", "mV", "mV", "V", "V", "V", "V", "V", "V", "V" };
        private readonly string[] STR_ANALOG_SMPMODE = { "ノーマル", "アベレージ" };
        private readonly double[] VALUE_ANALOG_RANGE = { 100, 200, 500, 1, 2, 5, 10, 20, 50, 100 };
        // ステータス
        private readonly string[] STR_LOG_STATUS = { "ロギング停止", "ロギング実行中" };
        private readonly string[] STR_OVERRUN_ERROR = { "エラーなし", "ハードバッファエラー", "ソフトバッファエラー" };
        #endregion  // 定数 終わり

        #region "変数"
        // pas1000クラス
        private pas1000 Pas1000w;
        #endregion  // 変数 終わり

        /// <summary>
        /// コンストラクタ。
        /// </summary>
        public pas1000s()
        {
            InitializeComponent();
        }

        /// <summary>
        /// フォームロード時の処理。
        /// </summary>
        /// <param name="sender">メッセージ送信元オブジェクト</param>
        /// <param name="e">イベント内容</param>
        private void pas1000s_Load(object sender, EventArgs e)
        {
            string szBuf;
            int i, j, num;

            ushort wDeviceNo;
            uint dwDrvVer, dwDllVer;
            pas1000.TDEVINFO DevInfo;
            string szVid, szPid;
            uint dwUnitId, dwNumCh, dwMaxNumCh;
            DateTime dt;
            int[] dwHostTime, dwFirmTime, dwFpgaTime;
            uint dwAdjStatus;
            pas1000.TSETMODE SetMode;
            string szCh;
            uint dwStartMode;
            uint dwSmpTime;
            string szSmpTime;
            uint dwHardNumData, dwSoftNumData, dwLogStatus, dwOverrunError;
            byte[] bLogBuf;
            int DispCh;
            byte[] bTmStampBuf;
            ulong dwTmStamp;
            byte bEv, bCh;
            string szTmStamp, szData;
            byte bInValue;

            // 変数を初期化
            Pas1000w = new pas1000();
            wDeviceNo = pas1000.DEVICE_AUTO;
            dwDrvVer = 0;
            dwDllVer = 0;
            DevInfo = new pas1000.TDEVINFO();
            DevInfo.bVendorID = new byte[pas1000.VID_LENGTH];
            DevInfo.bProductID = new byte[pas1000.PID_LENGTH];
            szVid = "";
            szPid = "";
            dwUnitId = 0;
            dwNumCh = 0;
            dwHostTime = new int[pas1000.DATE_TIME_LENGTH];
            dwFirmTime = new int[pas1000.DATE_TIME_LENGTH];
            dwFpgaTime = new int[pas1000.DATE_TIME_LENGTH];
            dwAdjStatus = 0;
            SetMode = new pas1000.TSETMODE();
            SetMode.Analog = new pas1000.TANALOG();
            SetMode.Analog.Ch = new pas1000.TANALOGCH[pas1000.ANALOG_NUM_CH];
            SetMode.Analog.StartTrig = new pas1000.TTRIG();
            SetMode.Analog.StopTrig = new pas1000.TTRIG();
            SetMode.Analog.dwStartTime = new uint[pas1000.START_TIME_LENGTH];
            SetMode.ExTrig = new pas1000.TEXTRIG[pas1000.EXTRIG_NUM_CH];
            SetMode.Alarm = new pas1000.TALARM();
            SetMode.Alarm.Analog = new pas1000.TALARMANALOG[pas1000.ANALOG_NUM_CH];
            SetMode.DigitalOut = new pas1000.TDIGITALOUT[pas1000.DIGITALOUT_NUM_CH];
            szCh = "";
            dwStartMode = 0;
            szSmpTime = "";
            dwHardNumData = 0;
            dwSoftNumData = 0;
            dwLogStatus = 0;
            dwOverrunError = 0;
            DispCh = -1;
            bTmStampBuf = new byte[8];
            szTmStamp = "";
            szData = "";
            bInValue = 0;

            //-------------------------------------------------------
            // デバイスの初期化処理
            //-------------------------------------------------------
            if (false == Pas1000w.Initialize(0, 0, -1))
            {
                ErrorMessage(wDeviceNo);
                goto END_PROCESS;
            }

            //-------------------------------------------------------
            // デバイスの検索
            //-------------------------------------------------------
            if (false == Pas1000w.Create(ref wDeviceNo))
            {
                ErrorMessage(wDeviceNo);
                goto END_PROCESS;
            }
            szBuf = string.Format("デバイス {0} の使用を宣言しました。", wDeviceNo);
            ResultMessage(szBuf);

            //-------------------------------------------------------
            // バージョン情報を取得
            //-------------------------------------------------------
            if (false == Pas1000w.GetVersion(ref dwDrvVer, ref dwDllVer))
            {
                ErrorMessage(wDeviceNo);
                goto END_PROCESS;
            }
            szBuf = string.Concat("<< バージョン情報 >>\n",
                                    string.Format("    ドライバ : {0:X8}H\n", dwDrvVer),
                                    string.Format("    DLL : {0:X8}H", dwDllVer));
            ResultMessage(szBuf);

            //-------------------------------------------------------
            // デバイス情報を取得
            //-------------------------------------------------------
            if (false == Pas1000w.GetDevInfo(wDeviceNo, ref DevInfo))
            {
                ErrorMessage(wDeviceNo);
                goto END_PROCESS;
            }
            for (i = 0; i < pas1000.VID_LENGTH; i++)
            {
                if (' ' == DevInfo.bVendorID[i]) break;
                szVid = string.Concat(szVid, (char)DevInfo.bVendorID[i]);
            }
            for (i = 0; i < pas1000.PID_LENGTH; i++)
            {
                if (' ' == DevInfo.bProductID[i]) break;
                szPid = string.Concat(szPid, (char)DevInfo.bProductID[i]);
            }
            szBuf = string.Concat("<< デバイス情報 >>\n",
                                    string.Format("    ベンダ ID : {0}\n", szVid),
                                    string.Format("    プロダクト ID : {0}\n", szPid),
                                    string.Format("    ファームバージョン : {0}.{1:D2}\n", DevInfo.bFirmVer / 100, DevInfo.bFirmVer % 100),
                                    string.Format("    FPGA バージョン : {0}.{1:D2}\n", DevInfo.bFpgaVer / 100, DevInfo.bFpgaVer % 100),
                                    string.Format("    USB 転送モード : {0}", STR_USB_MODE[DevInfo.bUsbMode]));
            ResultMessage(szBuf);

            //-------------------------------------------------------
            // ユニット情報を取得
            //-------------------------------------------------------
            if (false == Pas1000w.GetUnitInfo(wDeviceNo, ref dwUnitId, ref dwNumCh))
            {
                ErrorMessage(wDeviceNo);
                goto END_PROCESS;
            }
            dwMaxNumCh = (dwNumCh + 1) * 4;
            szBuf = string.Concat("<< ユニット情報 >>\n",
                                    string.Format("    ユニット ID : {0}\n", dwUnitId),
                                    string.Format("    アナログチャンネル数 : {0}CH", dwMaxNumCh));
            ResultMessage(szBuf);

            //-------------------------------------------------------
            // 時刻の設定
            //-------------------------------------------------------
            dt = DateTime.Now;                          // 現在の時刻を取得
            dwHostTime[0] = dt.Year - 2000;             // 年(下2桁)
            dwHostTime[1] = dt.Month;                   // 月
            dwHostTime[2] = dt.Day;                     // 日
            dwHostTime[3] = (int)dt.DayOfWeek;          // 曜
            dwHostTime[4] = dt.Hour;                    // 時
            dwHostTime[5] = dt.Minute;                  // 分
            dwHostTime[6] = dt.Second;                  // 秒
            if (false == Pas1000w.SetTime(wDeviceNo, dwHostTime))
            {
                ErrorMessage(wDeviceNo);
                goto END_PROCESS;
            }

            //-------------------------------------------------------
            // 時刻を取得
            //-------------------------------------------------------
            if (false == Pas1000w.GetTime(wDeviceNo, dwFirmTime, dwFpgaTime, ref dwAdjStatus))
            {
                ErrorMessage(wDeviceNo);
                goto END_PROCESS;
            }
            szBuf = string.Concat("<< 時刻 >>\n",
                                    string.Format("    ファーム : 20{0:D2}/{1:D2}/{2:D2}({3}) {4:D2}:{5:D2}:{6:D2}\n",
                                                    dwFirmTime[0], dwFirmTime[1], dwFirmTime[2], STR_WEEK[dwFirmTime[3]],
                                                    dwFirmTime[4], dwFirmTime[5], dwFirmTime[6]),
                                    string.Format("    FPGA : 20{0:D2}/{1:D2}/{2:D2} {3:D2}:{4:D2}:{5:D2}\n",
                                                    dwFpgaTime[0], dwFpgaTime[1], dwFpgaTime[2],
                                                    dwFpgaTime[4], dwFpgaTime[5], dwFpgaTime[6]));
            ResultMessage(szBuf);

            //-------------------------------------------------------
            // 動作モードの設定
            //-------------------------------------------------------
            // アナログ
            for (i = 0; i < pas1000.ANALOG_NUM_CH; i++)
            {
                if (dwMaxNumCh <= i) break;
                SetMode.Analog.Ch[i].dwChEnable = 1;    // チャンネル:有効
                SetMode.Analog.Ch[i].dwType = 0;        // 入力タイプ:電圧
                SetMode.Analog.Ch[i].dwRange = 6;       // 入力レンジ:±10V
            }
            SetMode.Analog.StartTrig.dwEnable = 0;      // スタートトリガ:無効
            SetMode.Analog.StopTrig.dwEnable = 0;       // ストップトリガ:無効
            SetMode.Analog.dwSamplingMode = 1;          // サンプリングモード:アベレージ
            SetMode.Analog.dwSamplingTime = 1000;       // サンプリングタイム:1ms
            SetMode.Analog.dwStartTimeEnable = 0;       // スタートタイム:無効
            SetMode.Analog.dwLoggingSize = 1000;        // ロギングサイズ:1000(1秒間)
            // 外部トリガ
            for (i = 0; i < pas1000.EXTRIG_NUM_CH; i++)
            {
                SetMode.ExTrig[i].dwInSlope = 0;        // 入力スロープ:↑
                SetMode.ExTrig[i].dwInFilter = 0;       // 入力フィルタ:0ms
            }
            // アラーム
            for (i = 0; i < pas1000.ANALOG_NUM_CH; i++)
            {
                if (dwMaxNumCh <= i) break;
                SetMode.Alarm.Analog[i].dwEnable = 0;   // アナログアラーム:無効
            }
            SetMode.Alarm.ExTrig.dwEnable = 0;          // 外部トリガアラーム:無効
            // デジタル出力
            for (i = 0; i < pas1000.DIGITALOUT_NUM_CH; i++)
            {
                SetMode.DigitalOut[i].dwType = 0;       // 出力タイプ:デジタル出力の設定値を反映
            }
            if (false == Pas1000w.SetMode(wDeviceNo, SetMode))
            {
                ErrorMessage(wDeviceNo);
                goto END_PROCESS;
            }
            for (i = 0; i < pas1000.ANALOG_NUM_CH; i++)
            {
                if (0 == SetMode.Analog.Ch[i].dwChEnable) continue;
                if ("" != szCh)
                {
                    szCh = string.Concat(szCh, "/");
                }
                szCh = string.Concat(szCh, string.Format("CH{0}", i + 1));

                if (-1 == DispCh)
                {
                    DispCh = i;                         // 結果表示用チャンネルを設定
                }
            }
            dwSmpTime = SetMode.Analog.dwSamplingTime;
            if (1000 > dwSmpTime)
            {
                szSmpTime = "μs";
            }
            else if (1000000 > dwSmpTime)
            {
                dwSmpTime /= 1000;
                szSmpTime = "ms";
            }
            else
            {
                dwSmpTime /= (1000 * 1000);
                szSmpTime = "s";
            }
            szBuf = string.Concat("<< 動作モード ( アナログ ) >>\n",
                                    string.Format("    チャンネル : {0}\n", szCh),
                                    string.Format("    入力タイプ : {0}\n", STR_ANALOG_TYPE[SetMode.Analog.Ch[DispCh].dwType]),
                                    string.Format("    入力レンジ : {0}\n", STR_ANALOG_RANGE[SetMode.Analog.Ch[DispCh].dwRange]),
                                    string.Format("    スタートトリガ : {0}\n", STR_ENABLE[SetMode.Analog.StartTrig.dwEnable]),
                                    string.Format("    ストップトリガ : {0}\n", STR_ENABLE[SetMode.Analog.StopTrig.dwEnable]),
                                    string.Format("    サンプリングモード : {0}\n", STR_ANALOG_SMPMODE[SetMode.Analog.dwSamplingMode]),
                                    string.Format("    サンプリングタイム : {0}{1}\n", dwSmpTime, szSmpTime),
                                    string.Format("    スタートタイム : {0}\n", STR_ENABLE[SetMode.Analog.dwStartTimeEnable]),
                                    string.Format("    ロギングサイズ : {0}", SetMode.Analog.dwLoggingSize));
            ResultMessage(szBuf);

            //-------------------------------------------------------
            // ロギングの開始
            //-------------------------------------------------------
            for (i = 0; i < pas1000.ANALOG_NUM_CH; i++)
            {
                if (dwMaxNumCh <= i) break;
                if (1 == SetMode.Analog.Ch[i].dwChEnable)
                {
                    dwStartMode |= (uint)(1 << i);
                }
            }
            szCh = "";
            for (i = 0; i < pas1000.ANALOG_NUM_CH; i++)
            {
                if (0 == (dwStartMode & (1 << i))) continue;
                if ("" != szCh)
                {
                    szCh = string.Concat(szCh, "/");
                }
                szCh = string.Concat(szCh, string.Format("CH{0}", i + 1));
            }
            szBuf = string.Concat("<< ロギング開始 >>\n",
                                    string.Format("    アナログチャンネル : {0}\n\n", szCh),
                                    " ※ロギングは自動的に停止します。\n",
                                    "    << ロギング停止 >> と表示されるまでお待ちください。");
            ResultMessage(szBuf);
            if (false == Pas1000w.StartLogging(wDeviceNo, dwStartMode))
            {
                ErrorMessage(wDeviceNo);
                goto END_PROCESS;
            }
            System.Threading.Thread.Sleep(1000);        // 1秒待ち

            // ロギング終了待ち
            while (true)
            {
                //---------------------------------------------------
                // チャンネル共通のステータスを取得
                //---------------------------------------------------
                if (false == Pas1000w.GetStatus(wDeviceNo, ref dwHardNumData, ref dwSoftNumData, ref dwLogStatus, ref dwOverrunError))
                {
                    ErrorMessage(wDeviceNo);
                    goto END_PROCESS;
                }
                // 全チャンネルのロギング停止で、ハードバッファのロギングデータ数が 0 の場合、ロギング終了
                if ((pas1000.STOP_LOGGING == dwLogStatus) && (0 == dwHardNumData)) break;
            }

            //-------------------------------------------------------
            // ロギングの停止
            //-------------------------------------------------------
            if (false == Pas1000w.StopLogging(wDeviceNo))
            {
                ErrorMessage(wDeviceNo);
                goto END_PROCESS;
            }
            ResultMessage("<< ロギング停止 >>");

            //-------------------------------------------------------
            // チャンネル共通のステータスを取得
            //-------------------------------------------------------
            if (false == Pas1000w.GetStatus(wDeviceNo, ref dwHardNumData, ref dwSoftNumData, ref dwLogStatus, ref dwOverrunError))
            {
                ErrorMessage(wDeviceNo);
                goto END_PROCESS;
            }
            szBuf = string.Concat("<< チャンネル共通のステータス >>\n",
                                    string.Format("    ハードバッファ : {0} ポイント\n", dwHardNumData),
                                    string.Format("    ソフトバッファ : {0} ポイント\n", dwSoftNumData),
                                    string.Format("    ロギングステータス : {0}\n", STR_LOG_STATUS[dwLogStatus]),
                                    string.Format("    オーバーランエラー : {0}", STR_OVERRUN_ERROR[dwOverrunError]));
            ResultMessage(szBuf);

            //-------------------------------------------------------
            // ロギングデータを取得
            //-------------------------------------------------------
            bLogBuf = new byte[dwSoftNumData * 3];
            if (false == Pas1000w.GetData(wDeviceNo, dwSoftNumData, bLogBuf))
            {
                ErrorMessage(wDeviceNo);
                goto END_PROCESS;
            }
            for (i = 0, j = 0, num = 1; i < bLogBuf.Length; i += 3)
            {
                bEv = (byte)(bLogBuf[i] & 0xE0);        // イベント情報
                bCh = (byte)(bLogBuf[i] & 0x0F);        // チャンネル情報

                // 表示用チャンネルのデータを取得
                if (DispCh == bCh)
                {
                    // タイムスタンプ
                    if (0 != bEv)
                    {
                        bTmStampBuf[j++] = bLogBuf[i + 1];
                        bTmStampBuf[j++] = bLogBuf[i + 2];
                    }
                    // データ
                    else
                    {
                        szData = string.Concat(szData,
                                                string.Format("       {0} : {1:X2}{2:X2}H", num, bLogBuf[i + 1], bLogBuf[i + 2]),
                                                string.Format(" ( {0:f5} )\n", VoltValue(SetMode.Analog.Ch[DispCh].dwRange,
                                                                                        ((uint)bLogBuf[i + 1] << 8) | (uint)bLogBuf[i + 2])));
                        if (10 == num) break;           // 10個分表示
                        num++;
                    }
                }
            }
            dwTmStamp = ((ulong)bTmStampBuf[0] << 56)
                            | ((ulong)bTmStampBuf[1] << 48)
                            | ((ulong)bTmStampBuf[2] << 40)
                            | ((ulong)bTmStampBuf[3] << 32)
                            | ((ulong)bTmStampBuf[4] << 24)
                            | ((ulong)bTmStampBuf[5] << 16)
                            | ((ulong)bTmStampBuf[6] << 8)
                            | (ulong)bTmStampBuf[7];
            szTmStamp = string.Format("{0:D2}/{1:D2}/{2:D2} {3:D2}:{4:D2}:{5:D2}.{6:D6}",
                                        (byte)((dwTmStamp >> 46) & 0x7F),
                                        (byte)((dwTmStamp >> 42) & 0x0F),
                                        (byte)((dwTmStamp >> 37) & 0x1F),
                                        (byte)((dwTmStamp >> 32) & 0x1F),
                                        (byte)((dwTmStamp >> 26) & 0x3F),
                                        (byte)((dwTmStamp >> 20) & 0x3F),
                                        (uint)(dwTmStamp & 0xFFFFF));
            szBuf = string.Concat("<< ロギングデータ >>\n",
                                    string.Format("    アナログチャンネル : CH{0}\n", DispCh + 1),
                                    string.Format("    タイムスタンプ : {0}\n\n", szTmStamp),
                                    string.Format("    データ [ {0} ポイント ]\n{1}\n", num, szData),
                                    string.Format("    ※カッコ内は電圧値 ( 単位 : {0} )",
                                                    STR_ANALOG_RANGE_UNIT[SetMode.Analog.Ch[DispCh].dwRange]));
            ResultMessage(szBuf);

            //-------------------------------------------------------
            // デジタル入力の読み込み
            //-------------------------------------------------------
            if (false == Pas1000w.DigitalIn(wDeviceNo, ref bInValue))
            {
                ErrorMessage(wDeviceNo);
                goto END_PROCESS;
            }
            szBuf = string.Concat("<< デジタル入力 >>\n",
                                    string.Format("    読み込みデータ : {0:X2}H", bInValue));
            ResultMessage(szBuf);

        END_PROCESS:
            //-------------------------------------------------------
            // デバイスの開放
            //-------------------------------------------------------
            if (pas1000.DEVICE_AUTO != wDeviceNo)
            {
                if (false == Pas1000w.Close(wDeviceNo))
                {
                    ErrorMessage(wDeviceNo);
                }
                szBuf = string.Format("デバイス {0} を開放しました。", wDeviceNo);
                ResultMessage(szBuf);
            }

            //-------------------------------------------------------
            // デバイスの終了処理
            //-------------------------------------------------------
            Pas1000w.Terminate();

            this.Close();
        }

        /// <summary>
        /// ロギングデータを電圧値に換算。
        /// </summary>
        /// <param name="dwRange">入力レンジ</param>
        /// <param name="dwData">ロギングデータ</param>
        /// <returns>電圧値</returns>
        private double VoltValue(uint dwRange, uint dwData)
        {
            double Vmax, Vmin;

            Vmax = VALUE_ANALOG_RANGE[dwRange];
            Vmin = -Vmax;
            return ((Vmax - Vmin) * dwData / DATA_RESOLUTION + Vmin);
        }

        /// <summary>
        /// 処理結果を表示。
        /// </summary>
        /// <param name="strMsg">表示するメッセージ文字列</param>
        private void ResultMessage(string strMsg)
        {
            MessageBox.Show(strMsg, this.Text, MessageBoxButtons.OK, MessageBoxIcon.Information);
        }

        /// <summary>
        /// エラーメッセージを表示。
        /// </summary>
        /// <param name="wDeviceNo">エラー情報を取得するデバイス識別番号</param>
        private void ErrorMessage(ushort wDeviceNo)
        {
            uint dwRes;
            string szBuf;

            dwRes = Pas1000w.GetLastError(wDeviceNo);
            szBuf = GetErrorStr(dwRes);
            MessageBox.Show(szBuf, this.Text, MessageBoxButtons.OK, MessageBoxIcon.Error);
        }

        /// <summary>
        /// エラー文字列を取得。
        /// </summary>
        /// <param name="dwRes">エラー番号</param>
        /// <returns>エラー文字列</returns>
        private string GetErrorStr(uint dwRes)
        {
            string szBuf;

            switch (dwRes & 0xFFFF)
            {
                case pas1000.ERR_SYSTEM:
                    return "システムエラーです。";
                case pas1000.ERR_NO_DEVICE:
                    return "使用可能なデバイスがありません。";
                case pas1000.ERR_IN_USE:
                    return "指定のデバイスは使用中です。";
                case pas1000.ERR_INVALID_DEVICENO:
                    return "不正なデバイス識別番号です。";
                case pas1000.ERR_INVALID_ARGUMENT:
                    szBuf = "不正な引数が指定されました。";
                    if (0 != (dwRes >> 16))
                    {
                        szBuf = string.Concat(szBuf, string.Format("\n[ Error code : {0:X4}H ]", dwRes >> 16));
                    }
                    return szBuf;
                case pas1000.ERR_INTERNAL:
                    return "内部エラーです。";
                case pas1000.ERR_TRANSFER_TIMEOUT:
                    return "データ転送がタイムアウトになりました。";
                case pas1000.ERR_TRANSFER_CANCEL:
                    return "データ転送がキャンセルされました。";
                case pas1000.ERR_TRANSFER_FAIL:
                    return "データ転送が失敗しました。";
                case pas1000.ERR_TRANSFER_SIZE:
                    return "データ転送のサイズが不正です。";
                case pas1000.ERR_HARDBUFF_OVERRUN:
                    return "ハードバッファがオーバーランしました。";
                case pas1000.ERR_SOFTBUFF_OVERRUN:
                    return "ソフトバッファがオーバーランしました。";
                case pas1000.ERR_DEVICE:
                    szBuf = "デバイスエラーです。";
                    if (0 != (dwRes >> 16))
                    {
                        szBuf = string.Concat(szBuf, string.Format("\n[ Error code : {0:X4}H ]", dwRes >> 16));
                    }
                    return szBuf;
                case pas1000.ERR_WRAPDLL:
                    return "DLL のロードに失敗しました。";
                default:
                    return string.Format("未定義のエラーです。\n[ Error code : {0:X8}H ]", dwRes);
            }
        }
    }
}
