﻿//===================================================================
//  pas1000w.cs
//
//  Copyright (C) P&A Technologies Inc.
//===================================================================
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Runtime.InteropServices;

namespace Sample
{
    #region "pas1000クラス"
    /// <summary>
    /// PA-S1000デバイス制御クラス。
    /// </summary>
    public class pas1000
    {
        #region "定数"
        //-----------------------------------------------------------
        // デバイスアクセスに関する定義
        //-----------------------------------------------------------
        public const uint MAX_DEVICE = 16;                          // 最大デバイス数
        public const ushort DEVICE_AUTO = 0xFFFF;                   // デバイス検索指定

        //-----------------------------------------------------------
        // チャンネルに関する定義
        //-----------------------------------------------------------
        public const int ANALOG_NUM_CH = 8;                         // アナログチャンネル数
        public const int EXTRIG_NUM_CH = 8;                         // 外部トリガチャンネル数
        public const int DIGITALOUT_NUM_CH = 4;                     // デジタル出力チャンネル数

        //-----------------------------------------------------------
        // デバイス情報に関する定義
        //-----------------------------------------------------------
        public const int VID_LENGTH = 8;                            // ベンダ識別文字列長
        public const int PID_LENGTH = 16;                           // プロダクト識別文字列長

        //-----------------------------------------------------------
        // 時間情報に関する定義
        //-----------------------------------------------------------
        public const int DATE_TIME_LENGTH = 7;                      // 日時の長さ(年/月/日(曜) 時:分:秒)
        public const int START_TIME_LENGTH = 5;                     // スタートタイムの長さ(日/時/分/秒/ミリ秒)

        //-----------------------------------------------------------
        // ステータスに関する定義
        //-----------------------------------------------------------
        public const uint STOP_LOGGING = 0x00;                      // ロギング停止
        public const uint RUNNING_LOGGING = 0x01;                   // ロギング実行中
        public const uint REACH_START_TIME = 0x02;                  // スタートタイムに達した
        public const uint PRE_START_TRIGPOS = 0x04;                 // スタートトリガポジションに達する前
        public const uint WAIT_START_TRIG = 0x08;                   // スタートトリガ待ち
        public const uint DETECT_START_TRIG = 0x10;                 // スタートトリガ検出
        public const uint PRE_STOP_TRIGPOS = 0x20;                  // ストップトリガポジションに達する前
        public const uint WAIT_STOP_TRIG = 0x40;                    // ストップトリガ待ち
        public const uint DETECT_STOP_TRIG = 0x80;                  // ストップトリガ検出

        //-----------------------------------------------------------
        // エラーコードの定義
        //-----------------------------------------------------------
        public const uint SUCCESS = 0x00;                           // 異常なし
        public const uint ERR_SYSTEM = 0x01;                        // システムエラー
        public const uint ERR_NO_DEVICE = 0x02;                     // 使用可能なデバイスがありません
        public const uint ERR_IN_USE = 0x03;                        // 指定のデバイスは使用中です
        public const uint ERR_INVALID_DEVICENO = 0x04;              // 不正なデバイス識別番号です
        public const uint ERR_INVALID_ARGUMENT = 0x05;              // 不正な引数が指定されました
        public const uint ERR_INTERNAL = 0x06;                      // 内部エラーです
        public const uint ERR_TRANSFER_TIMEOUT = 0x07;              // データ転送がタイムアウトになりました
        public const uint ERR_TRANSFER_CANCEL = 0x08;               // データ転送がキャンセルされました
        public const uint ERR_TRANSFER_FAIL = 0x09;                 // データ転送が失敗しました
        public const uint ERR_TRANSFER_SIZE = 0x0A;                 // データ転送のサイズが不正です
        public const uint ERR_HARDBUFF_OVERRUN = 0x0B;              // ハードバッファがオーバーランしました
        public const uint ERR_SOFTBUFF_OVERRUN = 0x0C;              // ソフトバッファがオーバーランしました
        public const uint ERR_DEVICE = 0x0D;                        // デバイスエラーです
        public const uint ERR_WRAPDLL = 0xFFFF;                     // DLL のロードに失敗しました

        public const uint ERR_ANALOG_SETTING = 0x01;                // アナログに関する設定で不正な引数が指定されました
        public const uint ERR_DIGITAL_SETTING = 0x04;               // デジタルに関する設定で不正な引数が指定されました
        public const uint ERR_ALARM_SETTING = 0x05;                 // アラームに関する設定で不正な引数が指定されました
        public const uint ERR_TRIGMODE_SETTING = 0x06;              // トリガモードに関する設定で不正な引数が指定されました
        public const uint ERR_EXTRIG_SETTING = 0x07;                // 外部トリガに関する設定で不正な引数が指定されました

        // アナログに関する設定で不正な引数が指定されました
        public const uint ERR_ANALOG_CH_ENABLE = 0x10;              // チャンネル使用状態
        public const uint ERR_ANALOG_TYPE = 0x11;                   // タイプ
        public const uint ERR_ANALOG_RANGE = 0x12;                  // レンジ
        public const uint ERR_ANALOG_BURNOUT = 0x13;                // バーンアウト
        public const uint ERR_ANALOG_COLDJUNCCOMP = 0x14;           // 冷接点補償
        public const uint ERR_ANALOG_SAMPLINGMODE = 0x15;           // サンプリングモード
        public const uint ERR_ANALOG_SAMPLINGTIME = 0x16;           // サンプリングタイム
        public const uint ERR_ANALOG_STARTTIME_ENABLE = 0x17;       // スタートタイム使用状態
        public const uint ERR_ANALOG_STARTTIME = 0x18;              // スタートタイム
        public const uint ERR_ANALOG_LOGGINGSIZE = 0x19;            // ロギングサイズ
        public const uint ERR_ANALOG_STARTTRIG_ENABLE = 0x1A;       // スタートトリガ使用状態
        public const uint ERR_ANALOG_STARTTRIG_CH = 0x1B;           // スタートトリガチャンネル
        public const uint ERR_ANALOG_STARTTRIG_MODE = 0x1C;         // スタートトリガモード
        public const uint ERR_ANALOG_STARTTRIG_SENSE = 0x1D;        // スタートトリガ検出モード
        public const uint ERR_ANALOG_STARTTRIG_LEVEL = 0x1E;        // スタートトリガレベル
        public const uint ERR_ANALOG_STARTTRIG_POS = 0x1F;          // スタートトリガポジション
        public const uint ERR_ANALOG_STOPTRIG_ENABLE = 0x20;        // ストップトリガ使用状態
        public const uint ERR_ANALOG_STOPTRIG_CH = 0x21;            // ストップトリガチャンネル
        public const uint ERR_ANALOG_STOPTRIG_MODE = 0x22;          // ストップトリガモード
        public const uint ERR_ANALOG_STOPTRIG_SENSE = 0x23;         // ストップトリガ検出モード
        public const uint ERR_ANALOG_STOPTRIG_LEVEL = 0x24;         // ストップトリガレベル
        public const uint ERR_ANALOG_STOPTRIG_POS = 0x25;           // ストップトリガポジション

        // 外部トリガに関する設定で不正な引数が指定されました
        public const uint ERR_EXTRIG_SLOPE = 0x30;                  // スロープ
        public const uint ERR_EXTRIG_FILTER = 0x31;                 // フィルタ

        // アラームに関する設定で不正な引数が指定されました
        public const uint ERR_ALARM_ANALOG_ENABLE = 0x40;           // アナログ使用状態
        public const uint ERR_ALARM_ANALOG_MODE = 0x41;             // アナログモード
        public const uint ERR_ALARM_ANALOG_SENSE = 0x42;            // アナログ検出モード
        public const uint ERR_ALARM_ANALOG_LEVEL = 0x43;            // アナログレベル
        public const uint ERR_ALARM_ANALOG_PORTOUT_ENABLE = 0x44;   // アナログポート出力状態
        public const uint ERR_ALARM_ANALOG_PORTOUT_CH = 0x45;       // アナログポート出力チャンネル
        public const uint ERR_ALARM_EXTRIG_ENABLE = 0x4A;           // 外部トリガ使用状態
        public const uint ERR_ALARM_EXTRIG_CH = 0x4B;               // 外部トリガチャンネル
        public const uint ERR_ALARM_EXTRIG_PORTOUT_ENABLE = 0x4C;   // 外部トリガポート出力状態
        public const uint ERR_ALARM_EXTRIG_PORTOUT_CH = 0x4D;       // 外部トリガポート出力チャンネル

        // デジタル出力に関する設定で不正な引数が指定されました
        public const uint ERR_DIGITALOUT_TYPE = 0x50;               // 出力タイプ
        public const uint ERR_DIGITALOUT_LEVEL = 0x51;              // 出力論理

        // デバイスエラー
        public const uint ERR_FPGA_SETTING = 0x10;                  // FPGA:設定処理が異常です
        public const uint ERR_RTC_INIT = 0x20;                      // RTC:イニシャライズ処理が異常です
        public const uint ERR_RTC_DATA_ACCESS = 0x21;               // RTC:データ転送アクセスが異常です
        public const uint ERR_RTC_DATA_SETTING = 0x22;              // RTC:データ設定が異常です
        public const uint ERR_RTC_TIME_SETTING = 0x23;              // RTC:時刻設定が異常です

        //-----------------------------------------------------------
        // メッセージの定義
        //-----------------------------------------------------------
        public const int WM_REMOVAL = 1;                            // デバイス抜去
        public const int WM_INSERT = 2;                             // デバイス挿入
        public const int WM_ERROR = 3;                              // データ取得スレッド内エラー

        //-----------------------------------------------------------
        // その他の定義
        //-----------------------------------------------------------
        public const uint DEVICE_ID = 0x5310;                       // デバイスID
        public const string DLL_NAME = "pas1000.dll";               // DLLファイル名
        #endregion  // 定数 終わり

        #region "デバイス情報構造体"
        [StructLayout(LayoutKind.Sequential)]
        public struct TDEVINFO
        {
            [MarshalAs(UnmanagedType.ByValArray, SizeConst = VID_LENGTH)]
            public byte[] bVendorID;                                // ベンダ識別文字列
            [MarshalAs(UnmanagedType.ByValArray, SizeConst = PID_LENGTH)]
            public byte[] bProductID;                               // プロダクト識別文字列
            public byte bFirmVer;                                   // ファームウェアのバージョン情報
            public byte bFpgaVer;                                   // FPGAのバージョン情報
            public byte bUsbMode;                                   // USB転送モード
        };
        #endregion  // デバイス情報構造体 終わり

        #region "動作モードパラメータ構造体"
        // トリガ機能構造体
        [StructLayout(LayoutKind.Sequential)]
        public struct TTRIG
        {
            public uint dwEnable;                                   // トリガ使用状態の設定
            public uint dwCh;                                       // トリガチャンネルの設定
            public uint dwMode;                                     // トリガモードの設定
            public uint dwSense;                                    // トリガ検出モードの設定
            public uint dwLevel0;                                   // トリガレベル0の設定
            public uint dwLevel1;                                   // トリガレベル1の設定
            public ulong dwPos;                                     // トリガポジションの設定
        };

        // アナログ機能構造体(チャンネル個別の設定)
        [StructLayout(LayoutKind.Sequential)]
        public struct TANALOGCH
        {
            public uint dwChEnable;                                 // アナログチャンネルの設定
            public uint dwType;                                     // アナログタイプの設定
            public uint dwRange;                                    // アナログ入力レンジの設定
            public uint dwBurnOut;                                  // アナログバーンアウトの設定
            public uint dwColdJuncComp;                             // アナログ冷接点補償の設定
        };

        // アナログ機能構造体
        [StructLayout(LayoutKind.Sequential)]
        public struct TANALOG
        {
            [MarshalAs(UnmanagedType.ByValArray, SizeConst = ANALOG_NUM_CH)]
            public TANALOGCH[] Ch;                                  // アナログチャンネル個別の設定
            public TTRIG StartTrig;                                 // アナログスタートトリガの設定
            public TTRIG StopTrig;                                  // アナログストップトリガの設定
            public uint dwSamplingMode;                             // アナログサンプリングモードの設定
            public uint dwSamplingTime;                             // アナログサンプリングタイムの設定
            public uint dwStartTimeEnable;                          // アナログスタートタイム使用状態の設定
            [MarshalAs(UnmanagedType.ByValArray, SizeConst = START_TIME_LENGTH)]
            public uint[] dwStartTime;                              // アナログスタートタイムの設定
            public ulong dwLoggingSize;                             // アナログロギングサイズの設定
        };

        // 外部トリガ構造体
        [StructLayout(LayoutKind.Sequential)]
        public struct TEXTRIG
        {
            public uint dwInSlope;                                  // 入力スロープ
            public uint dwInFilter;                                 // 入力フィルタ
        };

        // アラーム機能のアナログ構造体
        [StructLayout(LayoutKind.Sequential)]
        public struct TALARMANALOG
        {
            public uint dwEnable;                                   // アラーム使用状態の設定
            public uint dwMode;                                     // アラームモードの設定
            public uint dwSense;                                    // アラーム検出モードの設定
            public uint dwLevel0;                                   // アラームレベル0の設定
            public uint dwLevel1;                                   // アラームレベル1の設定
            public uint dwPortOutEnable;                            // アラームポート出力状態の設定
            public uint dwPortOutCh;                                // アラームポート出力チャンネルの設定
        };

        // アラーム機能の外部トリガ構造体
        [StructLayout(LayoutKind.Sequential)]
        public struct TALARMEXTRIG
        {
            public uint dwEnable;                                   // アラーム使用状態の設定
            public uint dwCh;                                       // アラームチャンネルの設定
            public uint dwPortOutEnable;                            // アラームポート出力状態の設定
            public uint dwPortOutCh;                                // アラームポート出力チャンネルの設定
        };

        // アラーム機能構造体
        [StructLayout(LayoutKind.Sequential)]
        public struct TALARM
        {
            [MarshalAs(UnmanagedType.ByValArray, SizeConst = ANALOG_NUM_CH)]
            public TALARMANALOG[] Analog;                           // アナログのアラーム要因の設定
            public TALARMEXTRIG ExTrig;                             // 外部トリガのアラーム要因の設定
        };

        // デジタル出力構造体
        [StructLayout(LayoutKind.Sequential)]
        public struct TDIGITALOUT
        {
            public uint dwType;                                     // デジタル出力タイプの設定
            public uint dwLevel;                                    // デジタル出力論理の設定
        };

        // 動作モード構造体
        [StructLayout(LayoutKind.Sequential)]
        public struct TSETMODE
        {
            public TANALOG Analog;                                  // アナログ機能
            [MarshalAs(UnmanagedType.ByValArray, SizeConst = EXTRIG_NUM_CH)]
            public TEXTRIG[] ExTrig;                                // 外部トリガ機能
            public TALARM Alarm;                                    // アラーム機能
            [MarshalAs(UnmanagedType.ByValArray, SizeConst = DIGITALOUT_NUM_CH)]
            public TDIGITALOUT[] DigitalOut;                        // デジタル出力
        };
        #endregion  // 動作モードパラメータ構造体 終わり

        #region "チャンネルごとのステータス構造体"
        // ロギングステータス構造体
        [StructLayout(LayoutKind.Sequential)]
        public struct TLOGGINGSTS
        {
            [MarshalAs(UnmanagedType.ByValArray, SizeConst = ANALOG_NUM_CH)]
            public uint[] dwAnalog;                                 // アナログ
        };

        // チャンネルステータス構造体
        [StructLayout(LayoutKind.Sequential)]
        public struct TCHSTATUS
        {
            public TLOGGINGSTS LoggingStatus;                       // ロギングステータス
            public uint dwAlarmStatus;                              // アラームステータス
        };
        #endregion  // チャンネルごとのステータス構造体 終わり

        #region "API"
        // 初期化処理
        [DllImport(DLL_NAME)]
        public static extern bool Pas1000Initialize(int hWnd, uint uWMBase, short TimeOut);

        // デバイスの検索
        [DllImport(DLL_NAME)]
        public static extern bool Pas1000Create(ref ushort pwDeviceNo);

        // デバイスの開放
        [DllImport(DLL_NAME)]
        public static extern bool Pas1000Close(ushort wDeviceNo);

        // 終了処理
        [DllImport(DLL_NAME)]
        public static extern bool Pas1000Terminate();

        // バージョン情報を取得
        [DllImport(DLL_NAME)]
        public static extern bool Pas1000GetVersion(ref uint pdwDrvVer, ref uint pdwDllVer);

        // デバイス情報を取得
        [DllImport(DLL_NAME)]
        public static extern bool Pas1000GetDevInfo(ushort wDeviceNo, IntPtr pDevInfo);

        // ユニットIDの設定
        [DllImport(DLL_NAME)]
        public static extern bool Pas1000SetUnitId(ushort wDeviceNo, uint dwUnitId);

        // ユニット情報を取得
        [DllImport(DLL_NAME)]
        public static extern bool Pas1000GetUnitInfo(ushort wDeviceNo, ref uint pdwUnitId, ref uint pdwNumCh);

        // 時刻の設定
        [DllImport(DLL_NAME)]
        public static extern bool Pas1000SetTime(ushort wDeviceNo, IntPtr pdwHostTime);

        // 時刻を取得
        [DllImport(DLL_NAME)]
        public static extern bool Pas1000GetTime(ushort wDeviceNo, IntPtr pdwFirmTime,
                                                    IntPtr pdwFpgaTime, ref uint pdwAdjustStatus);

        // 動作モードの設定
        [DllImport(DLL_NAME)]
        public static extern bool Pas1000SetMode(ushort wDeviceNo, IntPtr pSetMode);

        // デジタル入力の読み込み
        [DllImport(DLL_NAME)]
        public static extern bool Pas1000DigitalIn(ushort wDeviceNo, ref byte pbDigitalInValue);

        // デジタル出力の書き込み
        [DllImport(DLL_NAME)]
        public static extern bool Pas1000DigitalOut(ushort wDeviceNo, byte bDigitalOutValue);

        // アラームのクリア
        [DllImport(DLL_NAME)]
        public static extern bool Pas1000AlarmClear(ushort wDeviceNo, uint dwNumCh);

        // ロギングの開始
        [DllImport(DLL_NAME)]
        public static extern bool Pas1000StartLogging(ushort wDeviceNo, uint dwStartMode);

        // ロギングの停止
        [DllImport(DLL_NAME)]
        public static extern bool Pas1000StopLogging(ushort wDeviceNo);

        // チャンネル共通のステータスを取得
        [DllImport(DLL_NAME)]
        public static extern bool Pas1000GetStatus(ushort wDeviceNo, ref uint pdwHardNumData, ref uint pdwSoftNumData,
                                                    ref uint pdwLoggingStatus, ref uint pdwOverrunError);

        // チャンネルごとのステータスを取得
        [DllImport(DLL_NAME)]
        public static extern bool Pas1000GetChStatus(ushort wDeviceNo, IntPtr pChStatus);

        // ロギングデータを取得
        [DllImport(DLL_NAME)]
        public static extern bool Pas1000GetData(ushort wDeviceNo, uint dwNumData, IntPtr pbData);

        // エラーコードを取得
        [DllImportAttribute(DLL_NAME, SetLastError = true)]
        public static extern uint Pas1000GetLastError(ushort wDeviceNo);
        #endregion  // API 終わり

        /// <summary>
        /// 初期化処理。
        /// </summary>
        /// <param name="hWnd">DLLが送出するメッセージを受け取るウィンドウのハンドル。</param>
        /// <param name="uWMBase">送出するメッセージIDのベース値。</param>
        /// <param name="TimeOut">USB 転送完了待ちのタイムアウト時間。</param>
        /// <returns>成功 : true、失敗 : false。</returns>
        public bool Initialize(int hWnd, uint uWMBase, short TimeOut)
        {
            try
            {
                return Pas1000Initialize(hWnd, uWMBase, TimeOut);
            }
            catch (Exception)
            {
                return false;
            }
        }
        /// <summary>
        /// デバイスの検索。
        /// </summary>
        /// <param name="pwDeviceNo">使用するデバイスのデバイス識別番号を格納する領域へのポインタ。</param>
        /// <returns>成功 : true、失敗 : false。</returns>
        public bool Create(ref ushort pwDeviceNo)
        {
            try
            {
                return Pas1000Create(ref pwDeviceNo);
            }
            catch (Exception)
            {
                return false;
            }
        }
        /// <summary>
        /// デバイスの開放。
        /// </summary>
        /// <param name="wDeviceNo">対象デバイスのデバイス識別番号。</param>
        /// <returns>成功 : true、失敗 : false。</returns>
        public bool Close(ushort wDeviceNo)
        {
            try
            {
                return Pas1000Close(wDeviceNo);
            }
            catch (Exception)
            {
                return false;
            }
        }
        /// <summary>
        /// 終了処理。
        /// </summary>
        /// <returns>成功 : true、失敗 : false。</returns>
        public bool Terminate()
        {
            try
            {
                return Pas1000Terminate();
            }
            catch (Exception)
            {
                return false;
            }
        }
        /// <summary>
        /// バージョン情報を取得。
        /// </summary>
        /// <param name="pdwDrvVer">ドライバのバージョン情報を格納する領域へのポインタ。</param>
        /// <param name="pdwDllVer">DLLのバージョン情報を格納する領域へのポインタ。</param>
        /// <returns>成功 : true、失敗 : false。</returns>
        public bool GetVersion(ref uint pdwDrvVer, ref uint pdwDllVer)
        {
            try
            {
                return Pas1000GetVersion(ref pdwDrvVer, ref pdwDllVer);
            }
            catch (Exception)
            {
                return false;
            }
        }
        /// <summary>
        /// デバイス情報を取得。
        /// </summary>
        /// <param name="wDeviceNo">対象デバイスのデバイス識別番号。</param>
        /// <param name="pDevInfo">デバイス情報を格納する構造体へのポインタ。</param>
        /// <returns>成功 : true、失敗 : false。</returns>
        public bool GetDevInfo(ushort wDeviceNo, ref TDEVINFO pDevInfo)
        {
            try
            {
                bool res;
                IntPtr ptr;

                ptr = Marshal.AllocHGlobal(Marshal.SizeOf(pDevInfo));   // アンマネージメモリ確保
                Marshal.StructureToPtr(pDevInfo, ptr, false);           // マーシャリング(マネージメモリ->アンマネージメモリ)
                res = Pas1000GetDevInfo(wDeviceNo, ptr);
                pDevInfo = (TDEVINFO)(Marshal.PtrToStructure(ptr, typeof(TDEVINFO)));   // マーシャリング(アンマネージメモリ->マネージメモリ)
                Marshal.FreeHGlobal(ptr);                               // メモリを解放

                return res;
            }
            catch (Exception)
            {
                return false;
            }
        }
        /// <summary>
        /// ユニットIDの設定。
        /// </summary>
        /// <param name="wDeviceNo">対象デバイスのデバイス識別番号。</param>
        /// <param name="dwUnitId">ユニットID番号。</param>
        /// <returns>成功 : true、失敗 : false。</returns>
        public bool SetUnitId(ushort wDeviceNo, uint dwUnitId)
        {
            try
            {
                return Pas1000SetUnitId(wDeviceNo, dwUnitId);
            }
            catch (Exception)
            {
                return false;
            }
        }
        /// <summary>
        /// ユニット情報を取得。
        /// </summary>
        /// <param name="wDeviceNo">対象デバイスのデバイス識別番号。</param>
        /// <param name="pdwUnitId">ユニットIDを格納する領域へのポインタ。</param>
        /// <param name="pdwNumCh">ユニットに搭載されているアナログチャンネル数を格納する領域へのポインタ。</param>
        /// <returns>成功 : true、失敗 : false。</returns>
        public bool GetUnitInfo(ushort wDeviceNo, ref uint pdwUnitId, ref uint pdwNumCh)
        {
            try
            {
                return Pas1000GetUnitInfo(wDeviceNo, ref pdwUnitId, ref pdwNumCh);
            }
            catch (Exception)
            {
                return false;
            }
        }
        /// <summary>
        /// 時刻の設定。
        /// </summary>
        /// <param name="wDeviceNo">対象デバイスのデバイス識別番号。</param>
        /// <param name="pdwHostTime">日時を格納した領域へのポインタ。</param>
        /// <returns>成功 : true、失敗 : false。</returns>
        public bool SetTime(ushort wDeviceNo, int[] pdwHostTime)
        {
            try
            {
                bool res;
                int size;
                IntPtr ptr;

                size = Marshal.SizeOf(pdwHostTime[0]) * pdwHostTime.Length;
                ptr = Marshal.AllocHGlobal(size);                       // アンマネージメモリ確保
                Marshal.Copy(pdwHostTime, 0, ptr, pdwHostTime.Length);  // マネージメモリをアンマネージメモリにコピー
                res = Pas1000SetTime(wDeviceNo, ptr);
                Marshal.FreeHGlobal(ptr);                               // メモリを解放

                return res;
            }
            catch (Exception)
            {
                return false;
            }
        }
        /// <summary>
        /// 時刻を取得。
        /// </summary>
        /// <param name="wDeviceNo">対象デバイスのデバイス識別番号。</param>
        /// <param name="pdwFirmTime">ファーム時計の日時を格納する領域へのポインタ。</param>
        /// <param name="pdwFpgaTime">FPGA時計の日時を格納する領域へのポインタ。</param>
        /// <param name="pdwAdjustStatus">アジャストの状態を格納する領域へのポインタ。</param>
        /// <returns>成功 : true、失敗 : false。</returns>
        public bool GetTime(ushort wDeviceNo, int[] pdwFirmTime, int[] pdwFpgaTime, ref uint pdwAdjustStatus)
        {
            try
            {
                bool res;
                int size;
                IntPtr ptrFirm;
                IntPtr ptrFpga;

                size = Marshal.SizeOf(pdwFirmTime[0]) * pdwFirmTime.Length;
                ptrFirm = Marshal.AllocHGlobal(size);                       // アンマネージメモリ確保
                Marshal.Copy(pdwFirmTime, 0, ptrFirm, pdwFirmTime.Length);  // マネージメモリをアンマネージメモリにコピー
                size = Marshal.SizeOf(pdwFpgaTime[0]) * pdwFpgaTime.Length;
                ptrFpga = Marshal.AllocHGlobal(size);
                Marshal.Copy(pdwFpgaTime, 0, ptrFpga, pdwFpgaTime.Length);
                res = Pas1000GetTime(wDeviceNo, ptrFirm, ptrFpga, ref pdwAdjustStatus);
                Marshal.Copy(ptrFirm, pdwFirmTime, 0, pdwFirmTime.Length);  // アンマネージメモリをマネージメモリにコピー
                Marshal.Copy(ptrFpga, pdwFpgaTime, 0, pdwFpgaTime.Length);
                Marshal.FreeHGlobal(ptrFirm);                               // メモリを解放
                Marshal.FreeHGlobal(ptrFpga);

                return res;
            }
            catch (Exception)
            {
                return false;
            }
        }
        /// <summary>
        /// 動作モードの設定。
        /// </summary>
        /// <param name="wDeviceNo">対象デバイスのデバイス識別番号。</param>
        /// <param name="pSetMode">設定情報を格納している構造体のポインタ。</param>
        /// <returns>成功 : true、失敗 : false。</returns>
        public bool SetMode(ushort wDeviceNo, TSETMODE pSetMode)
        {
            try
            {
                bool res;
                IntPtr ptr;

                ptr = Marshal.AllocHGlobal(Marshal.SizeOf(pSetMode));   // アンマネージメモリ確保
                Marshal.StructureToPtr(pSetMode, ptr, false);           // マーシャリング(マネージメモリ->アンマネージメモリ)
                res = Pas1000SetMode(wDeviceNo, ptr);
                Marshal.FreeHGlobal(ptr);                               // メモリを解放

                return res;
            }
            catch (Exception)
            {
                return false;
            }
        }
        /// <summary>
        /// デジタル入力の読み込み。
        /// </summary>
        /// <param name="wDeviceNo">対象デバイスのデバイス識別番号。</param>
        /// <param name="pbDigitalInValue">デジタル入力を格納する領域へのポインタ。</param>
        /// <returns>成功 : true、失敗 : false。</returns>
        public bool DigitalIn(ushort wDeviceNo, ref byte pbDigitalInValue)
        {
            try
            {
                return Pas1000DigitalIn(wDeviceNo, ref pbDigitalInValue);
            }
            catch (Exception)
            {
                return false;
            }
        }
        /// <summary>
        /// デジタル出力の書き込み。
        /// </summary>
        /// <param name="wDeviceNo">対象デバイスのデバイス識別番号。</param>
        /// <param name="bDigitalOutValue">デジタル出力データ。</param>
        /// <returns>成功 : true、失敗 : false。</returns>
        public bool DigitalOut(ushort wDeviceNo, byte bDigitalOutValue)
        {
            try
            {
                return Pas1000DigitalOut(wDeviceNo, bDigitalOutValue);
            }
            catch (Exception)
            {
                return false;
            }
        }
        /// <summary>
        /// アラームのクリア。
        /// </summary>
        /// <param name="wDeviceNo">対象デバイスのデバイス識別番号。</param>
        /// <param name="dwNumCh">チャンネル番号。</param>
        /// <returns>成功 : true、失敗 : false。</returns>
        public bool AlarmClear(ushort wDeviceNo, uint dwNumCh)
        {
            try
            {
                return Pas1000AlarmClear(wDeviceNo, dwNumCh);
            }
            catch (Exception)
            {
                return false;
            }
        }
        /// <summary>
        /// ロギングの開始。
        /// </summary>
        /// <param name="wDeviceNo">対象デバイスのデバイス識別番号。</param>
        /// <param name="dwStartMode">開始モード。</param>
        /// <returns>成功 : true、失敗 : false。</returns>
        public bool StartLogging(ushort wDeviceNo, uint dwStartMode)
        {
            try
            {
                return Pas1000StartLogging(wDeviceNo, dwStartMode);
            }
            catch (Exception)
            {
                return false;
            }
        }
        /// <summary>
        /// ロギングの停止。
        /// </summary>
        /// <param name="wDeviceNo">対象デバイスのデバイス識別番号。</param>
        /// <returns>成功 : true、失敗 : false。</returns>
        public bool StopLogging(ushort wDeviceNo)
        {
            try
            {
                return Pas1000StopLogging(wDeviceNo);
            }
            catch (Exception)
            {
                return false;
            }
        }
        /// <summary>
        /// チャンネル共通のステータスを取得。
        /// </summary>
        /// <param name="wDeviceNo">対象デバイスのデバイス識別番号。</param>
        /// <param name="pdwHardNumData">ハードバッファのロギングデータ数を格納する領域へのポインタ。</param>
        /// <param name="pdwSoftNumData">ソフトバッファのロギングデータ数を格納する領域へのポインタ。</param>
        /// <param name="pdwLoggingStatus">ロギングステータスを格納する領域へのポインタ。</param>
        /// <param name="pdwOverrunError">オーバーランエラーを格納する領域へのポインタ。</param>
        /// <returns>成功 : true、失敗 : false。</returns>
        public bool GetStatus(ushort wDeviceNo, ref uint pdwHardNumData, ref uint pdwSoftNumData,
                                ref uint pdwLoggingStatus, ref uint pdwOverrunError)
        {
            try
            {
                return Pas1000GetStatus(wDeviceNo, ref pdwHardNumData, ref pdwSoftNumData,
                                                ref pdwLoggingStatus, ref pdwOverrunError);
            }
            catch (Exception)
            {
                return false;
            }
        }
        /// <summary>
        /// チャンネルごとのステータスを取得。
        /// </summary>
        /// <param name="wDeviceNo">対象デバイスのデバイス識別番号。</param>
        /// <param name="pChStatus">ステータス情報を格納する領域へのポインタ。</param>
        /// <returns>成功 : true、失敗 : false。</returns>
        public bool GetChStatus(ushort wDeviceNo, ref TCHSTATUS pChStatus)
        {
            try
            {
                bool res;
                IntPtr ptr;

                ptr = Marshal.AllocHGlobal(Marshal.SizeOf(pChStatus));  // アンマネージメモリ確保
                Marshal.StructureToPtr(pChStatus, ptr, false);          // マーシャリング(マネージメモリ->アンマネージメモリ)
                res = Pas1000GetChStatus(wDeviceNo, ptr);
                pChStatus = (TCHSTATUS)(Marshal.PtrToStructure(ptr, typeof(TCHSTATUS)));    // マーシャリング(アンマネージメモリ->マネージメモリ)
                Marshal.FreeHGlobal(ptr);                               // メモリを解放

                return res;
            }
            catch (Exception)
            {
                return false;
            }
        }
        /// <summary>
        /// ロギングデータを取得。
        /// </summary>
        /// <param name="wDeviceNo">対象デバイスのデバイス識別番号。</param>
        /// <param name="dwNumData">取得するデータ数。</param>
        /// <param name="pbData">ロギングデータを格納する領域へのポインタ。</param>
        /// <returns>成功 : true、失敗 : false。</returns>
        public bool GetData(ushort wDeviceNo, uint dwNumData, byte[] pbData)
        {
            try
            {
                bool res;
                int size;
                IntPtr ptr;

                size = Marshal.SizeOf(pbData[0]) * pbData.Length;
                ptr = Marshal.AllocHGlobal(size);               // アンマネージメモリ確保
                Marshal.Copy(pbData, 0, ptr, pbData.Length);    // マネージメモリをアンマネージメモリにコピー
                res = Pas1000GetData(wDeviceNo, dwNumData, ptr);
                Marshal.Copy(ptr, pbData, 0, pbData.Length);    // アンマネージメモリをマネージメモリにコピー
                Marshal.FreeHGlobal(ptr);                       // メモリを解放

                return res;
            }
            catch (Exception)
            {
                return false;
            }
        }
        /// <summary>
        /// エラーコードを取得。
        /// </summary>
        /// <param name="wDeviceNo">対象デバイスのデバイス識別番号。</param>
        /// <returns>エラーコード。</returns>
        public uint GetLastError(ushort wDeviceNo)
        {
            try
            {
                return Pas1000GetLastError(wDeviceNo);
            }
            catch (Exception)
            {
                return ERR_WRAPDLL;
            }
        }
    }
    #endregion  // pas1000クラス 終わり
}
